require 'sketchup.rb'


if( not file_loaded?("UVtools.rb") )

UI.add_context_menu_handler {|menu|
	ents=Sketchup.active_model.selection
	sel=Sketchup.active_model.selection.first
	sub=menu.add_submenu("UV Tools")
	sub.add_item("Spherical Map") {(UVtools.new.spherical_map(ents))}
	sub.add_item("Cylindrical Map") {(UVtools.new.cylindrical_map(ents))}
}

end
file_loaded("UVtools.rb")

class UVtools

@points=[]
@box=nil


######################
def get_bounds(ents)


all_points=[]
ents.each {|e|
	if e.class==Sketchup::Face
		all_points.push(e.vertices)
	end
}

return if all_points.empty?

all_points.flatten!
all_points.uniq!

box=Geom::BoundingBox.new
all_points.each {|v| box.add(v.position)}

@box=box

end

######################
def get_center(ents)  #returns the bounding box center of the selection

get_bounds(ents)
return @box.center

end

######################
def get_zmax_min(ents)  

get_bounds(ents)
return [@box.max.z,@box.min.z]

end
########################
def get_spherical_uv(p,c)  #p is the point we want to get the uvs for, c is the spherical center

pi=Math::PI
dx=p.x-c.x
dy=p.y-c.y
dz=p.z-c.z

theta=Math.acos(dz/(Math.sqrt(dx**2+dy**2+dz**2)))
phi=Math.atan2(dy,dx)

v=1.0-(theta/pi)

if phi<0.0 
	phi=2*pi+phi
end

u=(phi/(2*pi))

return [u,v,0.0]

end

########################
def get_cylindrical_uv(p,z_extents)  #p is the point we want to get the uvs for, ymin is the base of the cylinder

c=Geom::Point3d.new(@box.center)

pi=Math::PI
dx=p.x-c.x
dy=p.y-c.y

zmax=z_extents[0]
zmin=z_extents[1]
dz=(p.z-zmin)/(zmax-zmin)


phi=Math.atan2(dy,dx)

v=dz

if phi<0.0 
	phi=2*pi+phi
	#phi=-phi
end

u=phi/(2*pi)

return [u,v,0.0]

end

##########################
def validate_uvs(uvs)

u_arr=uvs.collect {|p| p.x}
u_max=u_arr.max
u_arr.each_index {|i|
	if u_arr[i]!=u_max
		if (u_max-u_arr[i])>0.8  #check for overlap
			#p "overlapped"
			u_arr[i]+=1.0
		end
	end
}

uvs.each_index {|i| uvs[i].x=u_arr[i]}  #modify the original uv array to fix the overlap

return uvs

end

##############################
def resize_uvs(uvs,mat)

tex=mat.texture
return if tex==nil

h=tex.height
w=tex.width

new_uvs=[]
uvs.each_index {|i|
	new_uvs[i]=Geom::Point3d.new(0,0,0)
	new_uvs[i].x=uvs[i].x*w
	new_uvs[i].y=uvs[i].y*h
	new_uvs[i].z=uvs[i].z
}

return new_uvs

end

 
#########################
def spherical_map(faces)  #performs spherical mapping

Sketchup.active_model.start_operation("Spherical Map")
faces=get_faces(faces)
origin=get_center(faces)
p origin

faces.each {|f|
	material=f.material
	verts=f.vertices
	pts=verts.collect {|v| v.position}
	uvs=pts.collect {|p| get_spherical_uv(p,origin)}
	uvs=validate_uvs(uvs)    #fixes the 'overlap' problem
	#uvs=resize_uvs(uvs,material)  #resize the uvs based on texture size
	if pts.length==3
		pt_array=[pts[0],uvs[0],pts[1],uvs[1],pts[2],uvs[2]]
	else
		pt_array=[pts[0],uvs[0],pts[1],uvs[1],pts[2],uvs[2],pts[3],uvs[3]]
	end
	begin
		f.position_material(material,pt_array,true)
		#f.position_material(material,pt_array,false)
	rescue
		p pt_array
		raise
	end
	
}

Sketchup.active_model.commit_operation
end	

#########################
def cylindrical_map(faces)  #performs spherical mapping

Sketchup.active_model.start_operation("Cylindrical Map")
faces=get_faces(faces)
z_extents=get_zmax_min(faces)

faces.each {|f|
	material=f.material
	verts=f.vertices
	pts=verts.collect {|v| v.position}
	uvs=pts.collect {|p| get_cylindrical_uv(p,z_extents)}
	uvs=validate_uvs(uvs)    #fixes the 'overlap' problem
	#uvs=resize_uvs(uvs,material)  #resize the uvs based on texture size
	if pts.length==3
		pt_array=[pts[0],uvs[0],pts[1],uvs[1],pts[2],uvs[2]]
	else
		pt_array=[pts[0],uvs[0],pts[1],uvs[1],pts[2],uvs[2],pts[3],uvs[3]]
	end
	begin
		f.position_material(material,pt_array,true)
		#f.position_material(material,pt_array,false)
	rescue
		p pt_array
		raise
	end
	
}

Sketchup.active_model.commit_operation
end	

############################
def get_faces(ents)

return ents.find_all {|e| e.class==Sketchup::Face}

end




end #class